@@ -15,6 +15,7 @@ from oml.lightning.entrypoints.parser import (
 from oml.lightning.modules.retrieval import RetrievalModule, RetrievalModuleDDP
 from oml.metrics.embeddings import EmbeddingMetrics, EmbeddingMetricsDDP
 from oml.registry.models import get_extractor_by_cfg
+from oml.registry.postprocessors import get_postprocessor_by_cfg
 from oml.registry.transforms import get_transforms_by_cfg
 from oml.utils.misc import dictconfig_to_dict
 
@@ -60,6 +61,11 @@ def pl_val(cfg: TCfg) -> Tuple[pl.Trainer, Dict[str, Any]]:
         **module_kwargs
     )
 
+    postprocessor = None if not cfg.get("postprocessor", None) else get_postprocessor_by_cfg(cfg["postprocessor"])
+
+    # Note! We add the link to our model to a Lightning's Module, so it can recognize it and manipulate its devices
+    pl_model.model_link_ = getattr(postprocessor, "model", None)
+
     metrics_constructor = EmbeddingMetricsDDP if is_ddp else EmbeddingMetrics
     metrics_calc = metrics_constructor(
         embeddings_key=pl_model.embeddings_key,
@@ -68,12 +74,13 @@ def pl_val(cfg: TCfg) -> Tuple[pl.Trainer, Dict[str, Any]]:
         is_query_key=valid_dataset.is_query_key,
         is_gallery_key=valid_dataset.is_gallery_key,
         extra_keys=(valid_dataset.paths_key, *valid_dataset.bboxes_keys),
+        postprocessor=postprocessor,
         **cfg.get("metric_args", {})
     )
     metrics_clb_constructor = MetricValCallbackDDP if is_ddp else MetricValCallback
     clb_metric = metrics_clb_constructor(
         metric=metrics_calc,
-        log_images=cfg.get("log_images", False),
+        log_images=False,
     )
 
     trainer = pl.Trainer(callbacks=[clb_metric], precision=cfg.get("precision", 32), **trainer_engine_params)
