@@ -5,6 +5,7 @@ from typing import Any, Collection, Dict, Iterable, List, Optional, Tuple, Union
 import matplotlib.pyplot as plt
 import numpy as np
 import torch
+from torch import Tensor
 
 from oml.const import (
     BLUE,
@@ -33,10 +34,11 @@ from oml.functional.metrics import (
     calc_gt_mask,
     calc_mask_to_ignore,
     calc_retrieval_metrics,
+    calc_topological_metrics,
     reduce_metrics,
 )
 from oml.interfaces.metrics import IMetricDDP, IMetricVisualisable
-from oml.interfaces.post_processor import IPostprocessor
+from oml.interfaces.retrieval import IDistancesPostprocessor
 from oml.metrics.accumulation import Accumulator
 from oml.utils.images.images import get_img_with_bbox, square_pad
 from oml.utils.misc import flatten_dict
@@ -44,6 +46,12 @@ from oml.utils.misc import flatten_dict
 TMetricsDict_ByLabels = Dict[Union[str, int], TMetricsDict]
 
 
+def validate_dataset(mask_gt: Tensor, mask_to_ignore: Tensor) -> None:
+    assert (
+        (mask_gt & ~mask_to_ignore).any(1).all()
+    ), "There are queries without available correct answers in the gallery!"
+
+
 class EmbeddingMetrics(IMetricVisualisable):
     """
     This class accumulates the information from the batches and embeddings produced by the model
@@ -66,10 +74,11 @@ class EmbeddingMetrics(IMetricVisualisable):
         cmc_top_k: Tuple[int, ...] = (5,),
         precision_top_k: Tuple[int, ...] = (5,),
         map_top_k: Tuple[int, ...] = (5,),
+        fmr_vals: Tuple[float, ...] = tuple(),
+        pfc_variance: Tuple[float, ...] = (0.5,),
         categories_key: Optional[str] = None,
-        postprocessor: Optional[IPostprocessor] = None,
+        postprocessor: Optional[IDistancesPostprocessor] = None,
         metrics_to_exclude_from_visualization: Iterable[str] = (),
-        check_dataset_validity: bool = True,
         return_only_main_category: bool = False,
         visualize_only_main_category: bool = True,
         verbose: bool = True,
@@ -82,15 +91,22 @@ class EmbeddingMetrics(IMetricVisualisable):
             is_query_key: Key to take the information whether every batch sample belongs to the query
             is_gallery_key: Key to take the information whether every batch sample belongs to the gallery
             extra_keys: Keys to accumulate some additional information from the batches
-            cmc_top_k: Values of ``k`` to compute ``CMC@k`` metrics
-            precision_top_k: Values of ``k`` to compute ``Precision@k`` metrics
-            map_top_k: Values of ``k`` to compute ``MAP@k`` metrics
+            cmc_top_k: Values of ``k`` to calculate ``cmc@k`` (`Cumulative Matching Characteristic`)
+            precision_top_k: Values of ``k`` to calculate ``precision@k``
+            map_top_k: Values of ``k`` to calculate ``map@k`` (`Mean Average Precision`)
+            fmr_vals: Values of ``fmr`` (measured in quantiles) to calculate ``fnmr@fmr`` (`False Non Match Rate
+                      at the given False Match Rate`).
+                      For example, if ``fmr_values`` is (0.2, 0.4) we will calculate ``fnmr@fmr=0.2``
+                      and ``fnmr@fmr=0.4``.
+                      Note, computing this metric requires additional memory overhead,
+                      that is why it's turned off by default.
+            pfc_variance: Values in range [0, 1]. Find the number of components such that the amount
+                          of variance that needs to be explained is greater than the percentage specified
+                          by ``pfc_variance``.
             categories_key: Key to take the samples' categories from the batches (if you have ones)
             postprocessor: Postprocessor which applies some techniques like query reranking
             metrics_to_exclude_from_visualization: Names of the metrics to exclude from the visualization. It will not
              affect calculations.
-            check_dataset_validity: Set ``True`` if you want to check if all the queries have valid answers in the
-             gallery set
             return_only_main_category: Whether you want to return only the main category from ``.compute_metrics()``
             visualize_only_main_category: Whether you want to visualize only the main category with ``.visualize()``
             verbose: Set ``True`` if you want to print metrics
@@ -104,6 +120,8 @@ class EmbeddingMetrics(IMetricVisualisable):
         self.cmc_top_k = cmc_top_k
         self.precision_top_k = precision_top_k
         self.map_top_k = map_top_k
+        self.fmr_vals = fmr_vals
+        self.pfc_variance = pfc_variance
 
         self.categories_key = categories_key
         self.postprocessor = postprocessor
@@ -111,28 +129,29 @@ class EmbeddingMetrics(IMetricVisualisable):
         self.distance_matrix = None
         self.mask_gt = None
         self.metrics = None
-        self.mask_to_ignore = None
+        self.metrics_unreduced = None
 
-        self.check_dataset_validity = check_dataset_validity
         self.visualize_only_main_category = visualize_only_main_category
         self.return_only_main_category = return_only_main_category
 
-        self.metrics_to_exclude_from_visualization = metrics_to_exclude_from_visualization
+        self.metrics_to_exclude_from_visualization = ["fnmr@fmr", "pcf", *metrics_to_exclude_from_visualization]
         self.verbose = verbose
 
-        self.keys_to_accumulate = [self.embeddings_key, self.is_query_key, self.is_gallery_key, self.labels_key]
+        keys_to_accumulate = [self.embeddings_key, self.is_query_key, self.is_gallery_key, self.labels_key]
         if self.categories_key:
-            self.keys_to_accumulate.append(self.categories_key)
+            keys_to_accumulate.append(self.categories_key)
         if self.extra_keys:
-            self.keys_to_accumulate.extend(list(extra_keys))
+            keys_to_accumulate.extend(list(extra_keys))
+        if self.postprocessor:
+            keys_to_accumulate.extend(self.postprocessor.needed_keys)
 
+        self.keys_to_accumulate = tuple(set(keys_to_accumulate))
         self.acc = Accumulator(keys_to_accumulate=self.keys_to_accumulate)
 
     def setup(self, num_samples: int) -> None:  # type: ignore
         self.distance_matrix = None
         self.mask_gt = None
         self.metrics = None
-        self.mask_to_ignore = None
 
         self.acc.refresh(num_samples=num_samples)
 
@@ -145,15 +164,20 @@ class EmbeddingMetrics(IMetricVisualisable):
         is_query = self.acc.storage[self.is_query_key]
         is_gallery = self.acc.storage[self.is_gallery_key]
 
-        if self.postprocessor:
-            # we have no this functionality yet
-            self.postprocessor.process()
-
-        # Note, in some of the datasets part of the samples may appear in both query & gallery.
+        # Note, in some datasets part of the samples may appear in both query & gallery.
         # Here we handle this case to avoid picking an item itself as the nearest neighbour for itself
-        self.mask_to_ignore = calc_mask_to_ignore(is_query=is_query, is_gallery=is_gallery)
-        self.mask_gt = calc_gt_mask(labels=labels, is_query=is_query, is_gallery=is_gallery)
-        self.distance_matrix = calc_distance_matrix(embeddings=embeddings, is_query=is_query, is_gallery=is_gallery)
+        mask_to_ignore = calc_mask_to_ignore(is_query=is_query, is_gallery=is_gallery)
+        mask_gt = calc_gt_mask(labels=labels, is_query=is_query, is_gallery=is_gallery)
+        distance_matrix = calc_distance_matrix(embeddings=embeddings, is_query=is_query, is_gallery=is_gallery)
+
+        self.distance_matrix, self.mask_gt = apply_mask_to_ignore(
+            distances=distance_matrix, mask_gt=mask_gt, mask_to_ignore=mask_to_ignore
+        )
+
+        validate_dataset(mask_gt=self.mask_gt, mask_to_ignore=mask_to_ignore)
+
+        if self.postprocessor:
+            self.distance_matrix = self.postprocessor.process_by_dict(self.distance_matrix, data=self.acc.storage)
 
     def compute_metrics(self) -> TMetricsDict_ByLabels:  # type: ignore
         if not self.acc.is_storage_full():
@@ -165,11 +189,13 @@ class EmbeddingMetrics(IMetricVisualisable):
 
         self._calc_matrices()
 
-        args = {
+        args_retrieval_metrics = {
             "cmc_top_k": self.cmc_top_k,
             "precision_top_k": self.precision_top_k,
             "map_top_k": self.map_top_k,
+            "fmr_vals": self.fmr_vals,
         }
+        args_topological_metrics = {"pfc_variance": self.pfc_variance}
 
         metrics: TMetricsDict_ByLabels = dict()
 
@@ -177,12 +203,14 @@ class EmbeddingMetrics(IMetricVisualisable):
         metrics[self.overall_categories_key] = calc_retrieval_metrics(
             distances=self.distance_matrix,
             mask_gt=self.mask_gt,
-            mask_to_ignore=self.mask_to_ignore,
-            check_dataset_validity=self.check_dataset_validity,
             reduce=False,
-            **args,  # type: ignore
+            mask_to_ignore=None,  # we already applied it
+            **args_retrieval_metrics,  # type: ignore
         )
 
+        embeddings = self.acc.storage[self.embeddings_key]
+        metrics[self.overall_categories_key].update(calc_topological_metrics(embeddings, **args_topological_metrics))
+
         if self.categories_key is not None:
             categories = np.array(self.acc.storage[self.categories_key])
             is_query = self.acc.storage[self.is_query_key]
@@ -194,12 +222,15 @@ class EmbeddingMetrics(IMetricVisualisable):
                 metrics[category] = calc_retrieval_metrics(
                     distances=self.distance_matrix[mask],  # type: ignore
                     mask_gt=self.mask_gt[mask],  # type: ignore
-                    mask_to_ignore=self.mask_to_ignore[mask],  # type: ignore
                     reduce=False,
-                    **args,  # type: ignore
+                    mask_to_ignore=None,  # we already applied it
+                    **args_retrieval_metrics,  # type: ignore
                 )
 
-        self.metrics_unreduced = metrics
+                mask = categories == category
+                metrics[category].update(calc_topological_metrics(embeddings[mask], **args_topological_metrics))
+
+        self.metrics_unreduced = metrics  # type: ignore
         self.metrics = reduce_metrics(metrics)  # type: ignore
 
         if self.return_only_main_category:
@@ -246,12 +277,7 @@ class EmbeddingMetrics(IMetricVisualisable):
         query_ids = self.get_worst_queries_ids(metric_name=metric_name, n_queries=n_queries)
         return self.get_plot_for_queries(query_ids=query_ids, n_instances=n_instances, verbose=verbose)
 
-    def get_plot_for_queries(
-        self,
-        query_ids: List[int],
-        n_instances: int,
-        verbose: bool = True,
-    ) -> plt.Figure:
+    def get_plot_for_queries(self, query_ids: List[int], n_instances: int, verbose: bool = True) -> plt.Figure:
         """
         Visualize the predictions for the query with the indicies <query_ids>.
 
@@ -263,8 +289,6 @@ class EmbeddingMetrics(IMetricVisualisable):
         """
         assert self.metrics is not None, "We are not ready to plot, because metrics were not calculated yet."
 
-        dist_matrix_with_inf, mask_gt = apply_mask_to_ignore(self.distance_matrix, self.mask_gt, self.mask_to_ignore)
-
         is_query = self.acc.storage[self.is_query_key]
         is_gallery = self.acc.storage[self.is_gallery_key]
 
@@ -289,11 +313,11 @@ class EmbeddingMetrics(IMetricVisualisable):
         query_bboxes = torch.tensor(bboxes)[is_query]
         gallery_bboxes = torch.tensor(bboxes)[is_gallery]
 
-        fig = plt.figure(figsize=(30, 30 / (n_instances + N_GT_SHOW_EMBEDDING_METRICS + 1) * len(query_ids)))
+        fig = plt.figure(figsize=(16, 16 / (n_instances + N_GT_SHOW_EMBEDDING_METRICS + 1) * len(query_ids)))
         for j, query_idx in enumerate(query_ids):
-            ids = torch.argsort(dist_matrix_with_inf[query_idx])[:n_instances]
+            ids = torch.argsort(self.distance_matrix[query_idx])[:n_instances]
 
-            n_gt = mask_gt[query_idx].sum()  # type: ignore
+            n_gt = self.mask_gt[query_idx].sum()  # type: ignore
 
             plt.subplot(
                 len(query_ids),
@@ -303,14 +327,16 @@ class EmbeddingMetrics(IMetricVisualisable):
 
             img = get_img_with_bbox(query_paths[query_idx], query_bboxes[query_idx], BLUE)
             img = square_pad(img)
+
             if verbose:
                 print("Q  ", query_paths[query_idx])
+
             plt.imshow(img)
             plt.title(f"Query, #gt = {n_gt}")
             plt.axis("off")
 
             for i, idx in enumerate(ids):
-                color = GREEN if mask_gt[query_idx, idx] else RED  # type: ignore
+                color = GREEN if self.mask_gt[query_idx, idx] else RED  # type: ignore
                 if verbose:
                     print("G", i, gallery_paths[idx])
                 plt.subplot(
@@ -320,11 +346,11 @@ class EmbeddingMetrics(IMetricVisualisable):
                 )
                 img = get_img_with_bbox(gallery_paths[idx], gallery_bboxes[idx], color)
                 img = square_pad(img)
-                plt.title(f"{i} - {round(dist_matrix_with_inf[query_idx, idx].item(), 3)}")
+                plt.title(f"{i} - {round(self.distance_matrix[query_idx, idx].item(), 3)}")
                 plt.imshow(img)
                 plt.axis("off")
 
-            gt_ids = mask_gt[query_idx].nonzero(as_tuple=True)[0][:N_GT_SHOW_EMBEDDING_METRICS]  # type: ignore
+            gt_ids = self.mask_gt[query_idx].nonzero(as_tuple=True)[0][:N_GT_SHOW_EMBEDDING_METRICS]  # type: ignore
 
             for i, gt_idx in enumerate(gt_ids):
                 plt.subplot(
@@ -334,7 +360,7 @@ class EmbeddingMetrics(IMetricVisualisable):
                 )
                 img = get_img_with_bbox(gallery_paths[gt_idx], gallery_bboxes[gt_idx], GRAY)
                 img = square_pad(img)
-                plt.title("GT " + str(round(dist_matrix_with_inf[query_idx, gt_idx].item(), 3)))
+                plt.title("GT " + str(round(self.distance_matrix[query_idx, gt_idx].item(), 3)))
                 plt.imshow(img)
                 plt.axis("off")
 
